
/*
   This file is part of AirFart.

   AirFart is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   AirFart is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with AirFart; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <pthread.h>
#include <semaphore.h>

#include <sys/time.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "interface.h"
#include "about.h"
#include "prefs.h"

#define LIST_VIEW_ICON_HEIGHT 15
#define LIST_VIEW_ICON_MAX_WIDTH 100

sem_t semaphore;
GtkWidget *treeview;
int active_secs;

/* menu items for the Show Columns menu */
GtkWidget *col_eth_addr_menu, *col_ssid_menu, *col_manufacturer_menu, *col_strength_menu, *col_strength_bar_menu, *col_active_menu, *col_packet_count_menu;
GtkTreeViewColumn *col_eth_addr, *col_ssid, *col_manufacturer, *col_strength, *col_strength_bar, *col_active, *col_packet_count;

GdkPixbuf* get_icon (char* file)
{

	GdkPixbuf *pixbuf = NULL;
	GdkPixbuf *pixbuf_icon = NULL;
	int width, height;
	double factor;

	pixbuf = gdk_pixbuf_new_from_file (file, NULL);

	if (pixbuf == NULL)
	{
		#ifdef DEBUG_INTERFACE
			printf("my %s pixfuf is null\n",file);
		#endif
		return NULL;
	}

	width = gdk_pixbuf_get_width (pixbuf);
	height = gdk_pixbuf_get_height (pixbuf);
	factor = (double) LIST_VIEW_ICON_HEIGHT / (double) height;
	pixbuf_icon = gdk_pixbuf_scale_simple (pixbuf, (int) (width * factor),
			LIST_VIEW_ICON_HEIGHT,
			GDK_INTERP_BILINEAR);

	g_object_unref (pixbuf);

	return pixbuf_icon;
}

GdkPixbuf* scale_icon_width (GdkPixbuf* pixbuf, int percentage)
{
	GdkPixbuf* pixbuf_icon;
	pixbuf_icon = gdk_pixbuf_scale_simple (pixbuf, (percentage * 2),
			LIST_VIEW_ICON_HEIGHT,
			GDK_INTERP_BILINEAR);

	g_object_unref (pixbuf);
	return pixbuf_icon;
}

enum
{
	COLUMN_ETH_ADDR_STRING,
	COLUMN_SSID_STRING,
	COLUMN_VENDOR_ID_STRING,
	COLUMN_STRENGTH,
	COLUMN_STRENGTH_BAR,
	COLUMN_PACKET_COUNT,
	COLUMN_ACTIVE,
	COLUMN_ACTIVE_LIGHT,
	COLUMN_TIME_STAMP,
	NUM_COLUMNS
};

void set_active_secs(GtkMenuItem *menuitem, gpointer user_data)
{
	active_secs = atoi( (char*)user_data );
}

int get_active_secs()
{
	return active_secs;
}

int deactivate ()
{

	GtkTreeIter iter;
	long time;
	glong time_stamp;
	gboolean active;
	struct timeval tv;
	GtkTreeModel* model;
	GdkPixbuf* light;
	model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));

		sem_wait(&semaphore);

		/* Critical Section */
		
		gettimeofday(&tv,NULL);
		time = tv.tv_sec;
		if (gtk_tree_model_get_iter_first (model, &iter)){//there is something in the tree
			do{
				gtk_tree_model_get(model, &iter, COLUMN_TIME_STAMP, &time_stamp, COLUMN_ACTIVE, &active, -1);

				// Should we deactivate this entry?
				if (active && (time_stamp + get_active_secs() <= time))
				{
					gtk_tree_model_get(model, &iter, COLUMN_ACTIVE_LIGHT, &light, -1);

					#ifdef DEBUG_INTERFACE
						printf("\n\n\n\nTurning off a light\n\n\n\n");
					#endif
					
					light = get_icon("/usr/share/pixmaps/airfart/offlight.png");
					gtk_list_store_set (GTK_LIST_STORE (model), &iter,
							COLUMN_ACTIVE_LIGHT, light,
							COLUMN_ACTIVE, FALSE,
							-1);//update the percentage
				}
			}while (gtk_tree_model_iter_next(model, &iter));
		}//end if

		/* End Critical Section */
		
		sem_post(&semaphore);
return 1;
}

static GtkTreeModel *create_model (void)
{
	//creates a new list store
	GtkListStore *store;
	store = gtk_list_store_new (NUM_COLUMNS,
			G_TYPE_STRING,
			G_TYPE_STRING,
			G_TYPE_STRING,
			G_TYPE_STRING,
			GDK_TYPE_PIXBUF,
			G_TYPE_STRING,
			G_TYPE_BOOLEAN,
			GDK_TYPE_PIXBUF,
			G_TYPE_LONG
			);
	return GTK_TREE_MODEL (store);
}


void update_treeview(unsigned char* eth_addr, char* ssid, char* vendor_id, gint strength, gint packet_count)
{
	#ifdef DEBUG_INTERFACE
		printf( "update_treeview called -- ssid: [%s] vendor: [%s] strength: %d\n", ssid, vendor_id, strength );
	#endif
	
	GdkPixbuf* strengthbar;
	GdkPixbuf* light;
	/* Used to create a new row in the tree view */
	gchar eth_addr_string[255];
	gchar ssid_string[255];

	long time;
	GtkTreeIter iter;
	gchar* eth_addr_gstr;
	GtkTreeModel *model;
	struct timeval tv;
	gettimeofday(&tv,NULL);
	time = tv.tv_sec;
	gboolean made_change = FALSE;
	gboolean active;
	model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
	light = get_icon("/usr/share/pixmaps/airfart/onlight.png");

	sem_wait(&semaphore);


	/* Critical Section */
	
	g_snprintf( eth_addr_string, 255, "%02x:%02x:%02x:%02x:%02x:%02x",
			eth_addr[0], eth_addr[1], eth_addr[2],
			eth_addr[3], eth_addr[4], eth_addr[5] );

	int ssid_len = strlen( ssid );
	g_snprintf( ssid_string, ssid_len+1, "%s", ssid); 

	// Is there something in the tree?
	if (gtk_tree_model_get_iter_first (model, &iter))
	{
		do
		{
			gtk_tree_model_get(model, &iter, COLUMN_ETH_ADDR_STRING, &eth_addr_gstr, -1);

			// Do we already have this ethernet address?
			if ( strcmp( eth_addr_gstr, eth_addr_string ) == 0 )
			{
				gtk_tree_model_get(model,&iter,COLUMN_STRENGTH_BAR,&strengthbar, COLUMN_ACTIVE, &active, -1);
				strengthbar = scale_icon_width(strengthbar,strength);

				gtk_list_store_set (GTK_LIST_STORE (model), &iter,
						COLUMN_STRENGTH, g_strdup_printf("%d%%", strength),
						COLUMN_STRENGTH_BAR,strengthbar,
						COLUMN_PACKET_COUNT, g_strdup_printf("%d", packet_count),
						COLUMN_ACTIVE, TRUE,
						COLUMN_TIME_STAMP, time,
						-1);
				if ( ! active )
				{
					gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_ACTIVE_LIGHT, light, -1);
				}
				
				made_change = TRUE;
			}
			
		} while ( ! made_change && gtk_tree_model_iter_next( model, &iter ) );
	}
	
	if ( ! made_change)
	{
		#ifdef DEBUG_INTERFACE
			printf("I still haven't found one yet\n");
		#endif

		strengthbar = get_icon("/usr/share/pixmaps/airfart/statusbar.png");
		strengthbar = scale_icon_width(strengthbar,strength);

		gtk_list_store_append (GTK_LIST_STORE (model), &iter);
		gtk_list_store_set (GTK_LIST_STORE (model), &iter,
				COLUMN_ETH_ADDR_STRING, eth_addr_string,
				COLUMN_SSID_STRING, ssid_string,
				COLUMN_VENDOR_ID_STRING, vendor_id,
				COLUMN_STRENGTH, g_strdup_printf("%d%%", strength),
				COLUMN_STRENGTH_BAR,strengthbar,
				COLUMN_PACKET_COUNT, g_strdup_printf("%d", packet_count),
				COLUMN_ACTIVE_LIGHT, light,
				COLUMN_ACTIVE, TRUE,
				COLUMN_TIME_STAMP, time,
				-1);
	}


        gtk_widget_draw(GTK_WIDGET(treeview), NULL);

	/* End Critical Section */

	sem_post(&semaphore);

}

void toggle_column( GtkMenuItem *menuitem, gpointer user_data )
{
	GtkTreeViewColumn *column;
	
	switch (atoi( (char*)user_data ))
	{
		case 1:
			column = col_eth_addr;
			break;
		case 2:
			column = col_ssid;
			break;
		case 3:
			column = col_manufacturer;
			break;
		case 4:
			column = col_strength;
			break;
		case 5:
			column = col_strength_bar;
			break;
		case 6:
			column = col_packet_count;
			break;
		case 7:
			column = col_active;
	}
	
	if (gtk_tree_view_column_get_visible(GTK_TREE_VIEW_COLUMN(column)))
		gtk_tree_view_column_set_visible (GTK_TREE_VIEW_COLUMN(column), FALSE);
	else
		gtk_tree_view_column_set_visible (GTK_TREE_VIEW_COLUMN(column), TRUE);	
}

static void add_columns ()
{
	GtkCellRenderer *renderer;

	
	renderer = gtk_cell_renderer_text_new ();
	col_eth_addr = gtk_tree_view_column_new_with_attributes ("Ethernet Address",
			renderer,
			"text",
			COLUMN_ETH_ADDR_STRING,
			NULL);
	/* set this column to a fixed sizing (of 110 pixels) */
	gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (col_eth_addr),
			GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width (GTK_TREE_VIEW_COLUMN (col_eth_addr), 110);
	gtk_tree_view_column_set_sort_column_id (col_eth_addr, COLUMN_ETH_ADDR_STRING);
	gtk_tree_view_append_column (GTK_TREE_VIEW(treeview), col_eth_addr);

	/* column for ssid*/
	renderer = gtk_cell_renderer_text_new ();
	col_ssid = gtk_tree_view_column_new_with_attributes ("SSID",
			renderer,
			"text",
			COLUMN_SSID_STRING,
			NULL);
	gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (col_ssid),
			GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width (GTK_TREE_VIEW_COLUMN (col_ssid), 150);
	gtk_tree_view_column_set_sort_column_id (col_ssid, COLUMN_SSID_STRING);
	gtk_tree_view_append_column (GTK_TREE_VIEW(treeview), col_ssid);

	/* column for vendor id*/
	renderer = gtk_cell_renderer_text_new ();
	col_manufacturer = gtk_tree_view_column_new_with_attributes ("Manufacturer",
			renderer,
			"text",
			COLUMN_VENDOR_ID_STRING,
			NULL);
	gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (col_manufacturer),
			GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width (GTK_TREE_VIEW_COLUMN (col_manufacturer), 150);
	gtk_tree_view_column_set_sort_column_id (col_manufacturer, COLUMN_VENDOR_ID_STRING);
	gtk_tree_view_append_column (GTK_TREE_VIEW(treeview), col_manufacturer);

	/* column for strength */
	renderer = gtk_cell_renderer_text_new ();
	col_strength = gtk_tree_view_column_new_with_attributes ("Strength",
			renderer,
			"text",
			COLUMN_STRENGTH,
			NULL);
	gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (col_strength),
			GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width (GTK_TREE_VIEW_COLUMN (col_strength), 60);
	gtk_tree_view_column_set_sort_column_id (col_strength, COLUMN_STRENGTH);
	gtk_tree_view_append_column (GTK_TREE_VIEW(treeview), col_strength);

	/* column for strength bar */
	renderer = gtk_cell_renderer_pixbuf_new ();
	g_object_set (G_OBJECT (renderer), "xalign", 0.0, NULL);//this makes the column aligned left
	col_strength_bar = gtk_tree_view_column_new_with_attributes ("Strength Bar",
			renderer,
			"pixbuf",
			COLUMN_STRENGTH_BAR,
			NULL);

	/* set this column to a fixed sizing of 100 pixels */
	gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (col_strength_bar),
			GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width (GTK_TREE_VIEW_COLUMN (col_strength_bar), 200);
	gtk_tree_view_column_set_sort_column_id (col_strength_bar, COLUMN_STRENGTH);
	gtk_tree_view_append_column (GTK_TREE_VIEW(treeview), col_strength_bar);
	
	/* Column for Packet Count */
	renderer = gtk_cell_renderer_text_new ();
	col_packet_count = gtk_tree_view_column_new_with_attributes ("Packet Count",
			renderer,
			"text",
			COLUMN_PACKET_COUNT,
			NULL);
	/* Set this column to be a fixed sizing of 100 pixels */
	gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (col_packet_count),
			GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width (GTK_TREE_VIEW_COLUMN (col_packet_count), 100);
	gtk_tree_view_column_set_sort_column_id (col_packet_count, COLUMN_PACKET_COUNT);
	gtk_tree_view_append_column (GTK_TREE_VIEW(treeview), col_packet_count); 
	
	/* column for active */
	renderer = gtk_cell_renderer_pixbuf_new ();
	col_active = gtk_tree_view_column_new_with_attributes ("Active",
			renderer,
			"pixbuf",
			COLUMN_ACTIVE_LIGHT,
			NULL);

	/* set this column to a fixed sizing of 20 pixels */
	gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (col_active),
			GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width (GTK_TREE_VIEW_COLUMN (col_active), 20);
	gtk_tree_view_column_set_sort_column_id (col_active, COLUMN_TIME_STAMP);
	gtk_tree_view_append_column (GTK_TREE_VIEW(treeview), col_active);
}

GtkWidget* create_main_window ()
{
	GtkWidget *main_window;
	GtkWidget *viewport1;
	GtkWidget *vbox1;
	GtkWidget *main_menu_bar;
	GtkWidget *file_menu_container, *file_menu, *prefs_menu, *file_menu_separator, *quit;
	GtkWidget *activation_menu_container, *activation_menu, *fields_menu, *fields_menu_container;
	GtkWidget *help_menu_container, *help_menu, *about;
	GtkWidget *scrolledwindow2;
	GtkWidget *statusbar;
	GtkAccelGroup *accel_group;
	GtkTreeModel *datamodel;
	GSList *active_group = NULL;
	GtkWidget *active_item;
	GtkWidget *about_image;
	pthread_t threadId;
	//ModelData* md;
	int err;
	gint i;

	sem_init(&semaphore,0,1);

	accel_group = gtk_accel_group_new ();

	main_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (main_window), "Airfart");

	viewport1 = gtk_viewport_new (NULL, NULL);
	gtk_widget_show (viewport1);
	gtk_container_add (GTK_CONTAINER (main_window), viewport1);

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox1);
	gtk_container_add (GTK_CONTAINER (viewport1), vbox1);

	main_menu_bar = gtk_menu_bar_new ();
	gtk_widget_show (main_menu_bar);
	gtk_box_pack_start (GTK_BOX (vbox1), main_menu_bar, FALSE, FALSE, 0);

	file_menu_container = gtk_menu_item_new_with_mnemonic ("_File");
	gtk_widget_show (file_menu_container);
	gtk_container_add (GTK_CONTAINER (main_menu_bar), file_menu_container);

	file_menu = gtk_menu_new ();
	gtk_menu_item_set_submenu (GTK_MENU_ITEM (file_menu_container), file_menu);

	prefs_menu = gtk_image_menu_item_new_from_stock("gtk-preferences", accel_group);
	gtk_widget_show(prefs_menu);
	gtk_container_add(GTK_CONTAINER(file_menu), prefs_menu);
	g_signal_connect (G_OBJECT(prefs_menu), "activate", G_CALLBACK (show_pref_window), NULL);
	
	file_menu_separator = gtk_separator_menu_item_new ();
	gtk_widget_show (file_menu_separator);
	gtk_container_add (GTK_CONTAINER (file_menu), file_menu_separator);
	gtk_widget_set_sensitive (file_menu_separator, FALSE);

	quit = gtk_image_menu_item_new_from_stock ("gtk-quit", accel_group);
	gtk_widget_show (quit);
	gtk_container_add (GTK_CONTAINER (file_menu), quit);
	g_signal_connect (G_OBJECT (quit), "activate", G_CALLBACK (destroy), NULL);

	/* Setup the activation menu */
	
	activation_menu_container = gtk_menu_item_new_with_mnemonic ("_Activation Time");
	gtk_widget_show (activation_menu_container);
	gtk_container_add (GTK_CONTAINER (main_menu_bar), activation_menu_container);

	activation_menu = gtk_menu_new ();
	gtk_menu_item_set_submenu (GTK_MENU_ITEM (activation_menu_container), activation_menu);

	for (i = 1; i <= 5; i++)
	{
		active_item = gtk_radio_menu_item_new_with_label (active_group, g_strdup_printf("%d second%c", i, ((i != 1) ? 's' : ' ')));
		active_group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (active_item));
		gtk_container_add (GTK_CONTAINER (activation_menu), active_item);
		gtk_signal_connect (GTK_OBJECT (active_item), "activate", GTK_SIGNAL_FUNC (set_active_secs), (gpointer) g_strdup_printf("%d", i));
		if (i == 3)
			gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (active_item), TRUE);
		gtk_widget_show(GTK_WIDGET(active_item));
	}
	
	/* Setup the Show Columns menu */
	fields_menu_container = gtk_menu_item_new_with_mnemonic ("_Show Columns");
	gtk_widget_show (fields_menu_container);
	gtk_container_add (GTK_CONTAINER (main_menu_bar), fields_menu_container);

	fields_menu = gtk_menu_new ();
	gtk_menu_item_set_submenu (GTK_MENU_ITEM (fields_menu_container), fields_menu);

	col_eth_addr_menu = gtk_check_menu_item_new_with_label ("Ethernet Address");
	gtk_widget_show (col_eth_addr_menu);
	gtk_container_add (GTK_CONTAINER (fields_menu), col_eth_addr_menu);
	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( col_eth_addr_menu ), TRUE );
	gtk_signal_connect (GTK_OBJECT (col_eth_addr_menu), "activate", GTK_SIGNAL_FUNC (toggle_column), (gpointer) "1");
	
	col_ssid_menu = gtk_check_menu_item_new_with_label ("SSID");
	gtk_widget_show (col_ssid_menu);
	gtk_container_add (GTK_CONTAINER (fields_menu), col_ssid_menu);
	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( col_ssid_menu ), TRUE );
	gtk_signal_connect (GTK_OBJECT (col_ssid_menu), "activate", GTK_SIGNAL_FUNC (toggle_column), (gpointer) "2");
	
	col_manufacturer_menu = gtk_check_menu_item_new_with_label ("Manufacturer");
	gtk_widget_show (col_manufacturer_menu);
	gtk_container_add (GTK_CONTAINER (fields_menu), col_manufacturer_menu);
	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( col_manufacturer_menu ), TRUE );
	gtk_signal_connect (GTK_OBJECT (col_manufacturer_menu), "activate", GTK_SIGNAL_FUNC (toggle_column), (gpointer) "3");

	col_strength_menu = gtk_check_menu_item_new_with_label ("Strength");
	gtk_widget_show(col_strength_menu);
	gtk_container_add (GTK_CONTAINER (fields_menu), col_strength_menu);
	gtk_check_menu_item_set_active ( GTK_CHECK_MENU_ITEM ( col_strength_menu ), TRUE );
	gtk_signal_connect (GTK_OBJECT (col_strength_menu), "activate", GTK_SIGNAL_FUNC (toggle_column), (gpointer) "4");
	
	col_strength_bar_menu = gtk_check_menu_item_new_with_label ("Strength Bar");
	gtk_widget_show (col_strength_bar_menu);
	gtk_container_add (GTK_CONTAINER (fields_menu), col_strength_bar_menu);
	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( col_strength_bar_menu ), TRUE );
	gtk_signal_connect (GTK_OBJECT (col_strength_bar_menu), "activate", GTK_SIGNAL_FUNC (toggle_column), (gpointer) "5");

	col_packet_count_menu = gtk_check_menu_item_new_with_label ("Packet Count");
	gtk_widget_show (col_packet_count_menu);
	gtk_container_add (GTK_CONTAINER (fields_menu), col_packet_count_menu);
	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM( col_packet_count_menu ), TRUE);
	gtk_signal_connect (GTK_OBJECT (col_packet_count_menu), "activate", GTK_SIGNAL_FUNC (toggle_column), (gpointer) "6");
	
	col_active_menu = gtk_check_menu_item_new_with_label ("Active Light");
	gtk_widget_show (col_active_menu);
	gtk_container_add (GTK_CONTAINER (fields_menu), col_active_menu);
	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( col_active_menu ), TRUE );
	gtk_signal_connect (GTK_OBJECT (col_active_menu), "activate", GTK_SIGNAL_FUNC (toggle_column), (gpointer) "7");

	/* Setup the help menu */	
	help_menu_container = gtk_menu_item_new_with_mnemonic ("_Help");
	gtk_widget_show (help_menu_container);
	gtk_container_add (GTK_CONTAINER (main_menu_bar), help_menu_container);

	help_menu = gtk_menu_new ();
	gtk_menu_item_set_submenu (GTK_MENU_ITEM (help_menu_container), help_menu);

	about_image = gtk_image_new_from_file("/usr/share/pixmaps/airfart/about_menu.png");
	about = gtk_image_menu_item_new_with_mnemonic ("_About");
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(about), GTK_WIDGET(about_image));
	gtk_widget_show (about);
	gtk_container_add (GTK_CONTAINER (help_menu), about);
	gtk_signal_connect_object(GTK_OBJECT(about), "activate",
			GTK_SIGNAL_FUNC(show_about), NULL);

	scrolledwindow2 = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (scrolledwindow2),GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
	gtk_widget_show (scrolledwindow2);
	gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow2, TRUE, TRUE, 0);

	datamodel = create_model();

	/* create tree view */
	treeview = gtk_tree_view_new_with_model (datamodel);

	#ifdef DEBUG_INTERFACE
		printf("i just set up my treeview\n");
	#endif

	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);

	g_object_unref (G_OBJECT (datamodel));

	gtk_container_add (GTK_CONTAINER (scrolledwindow2), treeview);
	add_columns();

	/* finish & show */
	gtk_signal_connect_object(GTK_OBJECT(main_window), "destroy", GTK_SIGNAL_FUNC(destroy), NULL);
	gtk_signal_connect_object(GTK_OBJECT(main_window), "delete_event", GTK_SIGNAL_FUNC(destroy), NULL);
			
	gtk_window_set_default_size (GTK_WINDOW (main_window), 820, 300);

	statusbar = gtk_statusbar_new ();
	gtk_widget_show (statusbar);
	gtk_box_pack_start (GTK_BOX (vbox1), statusbar, FALSE, FALSE, 0);

	gtk_window_add_accel_group (GTK_WINDOW (main_window), accel_group);

	if (!GTK_WIDGET_VISIBLE (main_window))
		gtk_widget_show_all (main_window);
	else
	{
		gtk_widget_destroy (main_window);
		main_window = NULL;
	}

	return main_window;
}

void destroy( GtkWidget *widget, gpointer data )
{
	gtk_main_quit ();
}
